C==============================================================================
C-----Fortran subroutines to read and write MatLab .mat files on format MatLab 5
C-----Use Routines provided by MatLab API libraries
C-----Pierre-Olivier Malaterre  - 5 may 2000
C==============================================================================
C-----On SGI the compilation command is:
C-----f77 -32 -mips2 rwmat5.f -col120 -L/local/apps/matlab5.3/extern/lib/sgi/ -lmat -lmx
C-----On peut peut etre aussi en 64 bits avec -64 -mips4 mais il semble alors qu'il manque
C-----une librairie au moins sur SGI: /usr/lib64/crt1.o


      SUBROUTINE MKM2( A, Name, N, NI, NJ, IOflag )
C-----A Fortran subroutine that writes a real double-precision matrix
C-----to a data file "slicotou.mat" in the .MAT-file format used by MATLAB 5
C-----A: matrix (double precision)
C-----Name: name of the matrix
C-----N: row dimension of A (max)
C-----NI: number of rows in A (active)
C-----NJ: number of columns in A (active)
C-----IOFlag = 1 for the first write (and open the slicotou.mat file)
C-----IOFlag = 0 for the next write
C-----IOFlag = -1 for the last write (and close the slicotou.mat file)
C
      PARAMETER(NMAX=200)
      DOUBLE PRECISION A(N,*),B(NMAX*NMAX)
      integer matOpen, mxCreateFull, mxCreateString 
      integer mp, pa1
      character Name*(*)
      integer N, NI, NJ, IOFlag
      save mp

      IF(IOFlag.EQ.1) THEN
C--------Open MAT-file for writing
C        write(16,*) 'Creating MAT-file slicotou.mat ...'
         mp = matOpen('slicotou.mat', 'w')
         if (mp .eq. 0) then
            write(16,*) 'Can''t open ''slicotou.mat'' for writing.'
            write(16,*) '(Do you have write permission in this directory?)'
            return
         end if
      ENDIF
C
C     Create variables
C
      pa1 = mxCreateFull(NI,NJ,0)
      call mxSetName(pa1, Name)
C
C     Transfer the matrix A into the vector B
C
      IF (NI*NJ.GT.NMAX*NMAX) THEN
         write(16,*) 'Matrix is too large',Name,NI,NJ
         return
      ENDIF
C
      K=0
      DO 20 J=1,NJ
         DO 10 I=1,NI
            K=K+1
            B(K)=A(I,J)
   10    CONTINUE
   20 CONTINUE
C
      I=NI*NJ
      call mxCopyReal8ToPtr(B, mxGetPr(pa1), I)
C     write(16,*) 'Write matrix in slicotou.mat:',Name,NI,NJ
      call matPutMatrix(mp, pa1)

C     clean up memory
      call mxFreeMatrix(pa1)
C
      IF(IOFLAG.EQ.-1) THEN
         status = matClose(mp)
         if (status .ne. 0) then
            write(16,*) 'Error closing MAT-file'
            return
         end if
      ENDIF
c
      end



      SUBROUTINE RDM2(A,Name,N,M,NI,NJ,IOFlag,rdflg)
c     A Fortran subroutine that reads a real double-precision matrix from a
c     data file (slicotin.mat) in the .MAT-file format used by MATLAB 5
c     Description of inputs:
c       Name: name of the matrix
c       N max number of rows in A
c       M max number of cols in A
c       IOFlag  : 1 Open the file slicotin.mat; 0 Normal; -1 Close the file
c     Description of outputs:
c       A       - matrix
c       NI      - number of rows in matrix
c       NJ      - number of columns in matrix
c       name    - character array holding the matrix name. (Be sure
c                 character array has room for the zero byte terminator)
c       pr      - real part of matrix (NI x NJ double precision
c                 elements stored column wise)
c       pi      - imaginary part of matrix
c       rdflg   - read status flag; = 0 good read, = 1 end of file,
c                 = -1 error reading file, = -2 dim problem
c
      PARAMETER(NMAX=200)
      DOUBLE PRECISION A(N,*),B(NMAX*NMAX)
      integer  NI, NJ
      character  Name*(*)
      integer  rdflg

      integer matOpen, matGetDir, matGetNextMatrix, matGetFull
      integer mp, dir, adir(100), pa
C----------------------------------------------------------------------
C
C     Other variable declarations here
C     
      integer   mxGetM, mxGetN, matClose
      integer   ndir, i, stat
      integer   pr, pi
      character*32 names(100), mxGetName
      save mp

C
C-------------------------------------------------------------
C     Open file and read directory
C-------------------------------------------------------------
C
      IF(IOFlag.EQ.1) THEN
         rdflg = 0
         mp = matOpen('slicotin.mat', 'r')
C
         if (mp .eq. 0) then
            write(16,*) 'Can''t open ''slicotin.mat''.'
            rdflg = -1
            return
         end if
C
C        Read directory
C
c        dir = matgetdir(mp, ndir)
c        if (dir .eq. 0) then
c           write(16,*) 'Can''t read directory.'
c           return
c        endif
C
C        Copy  integer into an array of pointers
C
c        call  mxCopyPtrToInteger4 ( dir , adir , ndir )
C
C        Copy  integer to character string
C
c        do 10 i=1,ndir
c           call mxCopyPtrToCharacter(adir(i), names(i), 32)
c  10    continue
C
C        write(16,*) 'Directory of Mat-file:'
C        do 20 i=1,ndir
C           write(16,*) names(i)
C  20    continue
      ENDIF
C
C-------------------------------------------------------------
C     Read full arrays
C-------------------------------------------------------------
C
      rdflg = 0
C     write(16,*) 'Getting Matrix: ',Name
      pa = matGetFull(mp,Name,NI,NJ,pr,pi)

      IF (pa .ne. 0) THEN
         write(16,*) 'Cannot get Matrix: ',Name
         rdflg = -1
         return
      ENDIF

C-----management of problems on dimensions
C-----in this case only the possible sub matrix will be read
      if (NI.GT.N) then
         WRITE(16,*) 'Error: matrix too large in .mat file (rows)'
         rdflg = -2
      endif
      if (NJ.GT.M) then
         WRITE(16,*) 'Error: matrix too large in .mat file (columns)'
         rdflg = -2
      endif

      call mxCopyPtrToReal8(pr, B, NI*NJ)

      K=0
      DO 40 J=1,NJ
         DO 30 I=1,NI
            K=K+1
            A(I,J)=B(K)
   30    CONTINUE
   40 CONTINUE
C
      call mxFree(pr)
      call mxFree(pi)

      IF(IOFLAG.EQ.-1) THEN
         stat = matClose(mp)
         if (stat .ne. 0) then
            write(16,*) 'Error closing ''slicotin.mat''.'
            return
         end if
      ENDIF

      end
