function [Num,Den] = TB03AD(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,Arg11,Arg12)
%Different syntaxes are allowed to call this function:
%function [Num,Den] = TB03AD(A,B,C,D,LERI,TYPE,FLAG,SCALING,TOL,EQUIL,OUTP,CLEAN)
%function [Num,Den] = TB03AD(A,B,C,D,LERI,TYPE,FLAG,SCALING)
%function [Num,Den] = TB03AD(A,B,C,D,LERI,TYPE,FLAG)
%function [Num,Den] = TB03AD(A,B,C,D,LERI,TYPE,FLAG)
%function [Num,Den] = TB03AD(A,B,C,D,LERI,TYPE)
%function [Num,Den] = TB03AD(A,B,C,D,LERI)
%function [Num,Den] = TB03AD(A,B,C,D)
%function [Num,Den] = TB03AD(Gsys,LERI,TYPE,FLAG,SCALING,TOL,EQUIL,OUTP,CLEAN)
%function [Num,Den] = TB03AD(Gsys,LERI,TYPE,FLAG,SCALING)
%function [Num,Den] = TB03AD(Gsys,LERI,TYPE,FLAG)
%function [Num,Den] = TB03AD(Gsys,LERI,TYPE,FLAG)
%function [Num,Den] = TB03AD(Gsys,LERI,TYPE)
%function [Num,Den] = TB03AD(Gsys,LERI)
%function [Num,Den] = TB03AD(Gsys)
%
%The system can be provided as a system (Gsys: tf or ss)
%     or by the (A,B,C,D) matrices of its state space reprensentation
%Some parameters are optionnal (LERI,TYPE,FLAG,SCALING,TOL,EQUIL,OUTP,CLEAN)
%     If not provided the default variables are:
%     LERI='R',TYPE='s',FLAG='m3',SCALING=0,TOL=0,EQUIL='N',OUTP=1,CLEAN=1
%
%     This TB03AD.m MatLab function and the MTB03AD.exe program has been written by
%     Pierre-Olivier Malaterre (Cemagref, ISU) March 2000
%     from the SLICOT subroutine TB03AD
%     The SLICOT subroutines are based on LAPACK and BLAS subroutines
%     All FORTRAN source codes of these subroutines (BLAS, LAPACK, SLICOT) can be found on Internet
%
%     Modified 3/28/2000 by POM to add the scaling and clean options
%     Modified 4/07/2000 by POM to allow multiple call syntaxes + change FLAG to 'm3' or 'tf'
%                                                               + change output to [Num,Den] 
%     Modified 4/28/2000 by POM to add another scaling option
%
%     PURPOSE
%
%     To find a relatively prime left polynomial matrix representation 
%     inv(P(s))*Q(s) or right polynomial matrix representation
%     Q(s)*inv(P(s)) with the same transfer matrix T(s) as that of a 
%     given state-space representation, i.e.
%
%        inv(P(s))*Q(s) = Q(s)*inv(P(s)) = T(s) = C*inv(s*I-A)*B + D.
%
%     PROGRAMING PRINCIPLE
%
%     A, B, C, D matrices are saved in the slicotin.mat file in MatLab 4 format (-V4 OPTION)
%     Other parameters are saved in the mtb03ad.dat ASCII file
%     The MTB03AD.exe program is then called (! command)
%     This program has been compiled with the MICROSOFT FORTRAN POWERSTATION 4.0 COMPILER
%     This program reads both files, check the data, calls the SLICOT TB03AD routine
%     then processes and writes the result data on the slicotou.mat file in MatLab 4 format
%     The TB03AD.M function then further processes the results providing P and Q polynomials
%     These polynomials can be provided as TF systems (in s, z or z^-1 form) or as arrays of matrices
%     Some additionnal results and messages are also saved on the mtb03ad.res ASCII file
%     The slicotou.mat file is not deleted (if CLEAN = 0) and therefore Pi and Qi matrices
%     can also be read from it
%
%     LIMITATIONS
%     These maxima can be extended but the MTB03AD.f program must be re-compiled:
%
%     N: nb of states (NMAX = 200)
%     M: nb of inputs (MMAX = 50)
%     P: nb of outputs (PMAX = 50)
%
%     MANAGEMENT OF ERROR
%     Several tests are carried out either in this TB03AD.M MatLab routine or in the MTB03AD.EXE program
%     In case a problem is detected in MTB03AD.EXE program, messages are written on the mtb03ad.res file
%     To be able to read this file you must set CLEAN=0
%
%     ARGUMENTS
%
%     A,B,C,D MATRICES
%             state-space representation (continuous or discrete, not important)
%     or
%
%     Gsys    SYSTEM
%             in tf or ss format (continuous or discrete, not important)
%
%     LERI    CHARACTER*1
%             Indicates whether the left polynomial matrix
%             representation or the right polynomial matrix
%             representation is required as follows:
%             = 'L':  A left matrix fraction is required
%             = 'R':  A right matrix fraction is required (default)
%
%     TYPE    CHARACTER*1
%             specifies the type of polynomial P and Q provided
%             = 's' (default) or 'z' for:
%             {matrices Pi are provided for increasing powers}
%             P = P0 + P1 s + P2 s^2 + ... + Pn s^n       or
%             P = P0 + P1 z + P2 z^2 + ... + Pn z^n
%             = 'l' or 'q' (ie z^-1) for:
%             {matrices Pi are provided for decreasing powers}
%             {this second form is useful to get lambda transform after simplification between Q and P}
%             P = P0 z^n + P1 z^(n-1) + ... + P(n-1) z + P(n)
%               = z^n [P0 + P1 z^(-1) + ... + P(n-1) z^(-n+1) + P(n) z^(-n)]
%               = q^(-n) [P0 + P1 q^(1) + ... + P(n-1) q^(n-1) + P(n) q^(n)]
%
%     FLAG    CHARACTER*2
%             specifies the type of output
%             = 'tf' : Num=Q and Den=P are provided as sys tf object
%             = 'm3' : Num=Q and Den=P are provided as arrays of matrices (default)
%
%     SCALING INTEGER
%             specifies if the output polynomials are scaled
%             = 0 : no scaling (default)
%             = 1 : P and Q are scaled by Ps=1/max(max(abs(P0)))
%             = 2 : P and Q are scaled by Ps=1/max(max(max(abs(Pi)))) (works better)
%             = 3 : P and Q are scaled by Ps=inv(P0) (condition problems sometimes)
%             for LERI = 'L':
%             P = P0*Ps + P1*Ps q^(1) + ... + P(n-1)*Ps q^(n-1) + P(n)*Ps q^(n)
%             Q = Ps*Q0 + Ps*Q1 q^(1) + ... + Ps*Q(n-1) q^(n-1) + Ps*Q(n) q^(n)
%             for LERI = 'R':
%             P = Ps*P0 + Ps*P1 q^(1) + ... + Ps*P(n-1) q^(n-1) + Ps*P(n) q^(n)
%             Q = Q0*Ps + Q1*Ps q^(1) + ... + Q(n-1)*Ps q^(n-1) + Q(n)*Ps q^(n)
%
%     TOL     DOUBLE PRECISION (default=0)
%             The tolerance to be used in rank determination when 
%             transforming (A, B, C). If the user sets TOL > 0, then
%             the given value of TOL is used as a lower bound for the
%             reciprocal condition number (see the description of the
%             argument RCOND in the SLICOT routine MB03OD);  a 
%             (sub)matrix whose estimated condition number is less than
%             1/TOL is considered to be of full rank.  If the user sets
%             TOL <= 0, then an implicitly computed, default tolerance
%             (determined by the SLICOT routine TB01UD) is used instead.
%
%     EQUIL   CHARACTER*1
%             Specifies whether the user wishes to balance the triplet
%             (A,B,C), before computing a minimal state-space
%             representation, as follows:
%             = 'S':  Perform balancing (scaling)
%             = 'N':  Do not perform balancing (default)
%
%     OUTP    INTEGER
%             = 1:  writes polynomials results only on slicotou.mat (default)
%             = 2:  writes polynomials results on slicotou.mat and mtb03ad.res
%
%     CLEAN   INTEGER
%             Specifies if temporary files are deleted
%             = 0 : no file is deleted
%             = 1 : the slicotin.mat, mtb03ad.dat, slicotou.mat, mtb03ad.res files are deleted (default)

% Set default values
LERI = 'R';
TYPE = 's';
FLAG = 'm3';
SCALING = 0;
TOL = 0;
EQUIL = 'N';
OUTP = 1;
CLEAN = 1;

% Verification of input variables for System
if isa(Arg1,'double')
   A=Arg1;
   B=Arg2;
   C=Arg3;
   D=Arg4;
   shift=4;
elseif isa(Arg1,'ss') | isa(Arg1,'tf')
   [A,B,C,D]=ssdata(Arg1);
   shift=1;
else
   error('syntax error');
end

% Set other parameters if provided
if nargin - shift > 0
   LERI = eval(strcat('Arg',num2str(shift+1)));
end
if nargin - shift > 1
   TYPE = eval(strcat('Arg',num2str(shift+2)));
end
if nargin - shift > 2
   FLAG = eval(strcat('Arg',num2str(shift+3)));
end
if nargin - shift > 3
   SCALING = eval(strcat('Arg',num2str(shift+4)));
end
if nargin - shift > 4
   TOL = eval(strcat('Arg',num2str(shift+5)));
end
if nargin - shift > 5
   EQUIL = eval(strcat('Arg',num2str(shift+6)));
end
if nargin - shift > 6
   OUTP = eval(strcat('Arg',num2str(shift+7)));
end
if nargin - shift > 7
   CLEAN = eval(strcat('Arg',num2str(shift+8)));
end

% Test if values are valid
if LERI ~= 'R' & LERI ~= 'L'
   error('syntax error for LERI');
end
if TYPE == 's' | TYPE == 'z'
   % Increasing powers
   TYPE2 = 'I';
elseif TYPE == 'l' | TYPE == 'q'
   % Decreasing powers
   TYPE2 = 'D';
else
   error('syntax error for TYPE');
end
if FLAG ~= 'tf' & FLAG ~= 'm3'
   error('syntax error for FLAG');
end
if SCALING < 0 | SCALING > 3
   error('syntax error for SCALING');
end
if EQUIL ~= 'S' & EQUIL ~= 'N'
   error('syntax error for EQUIL');
end
if OUTP ~= 1 & OUTP ~= 2
   error('syntax error for OUTP');
end
if CLEAN ~= 0 & CLEAN ~= 1
   error('syntax error for CLEAN');
end

% save matrices on the slicotin.mat file for slicot subroutine
if strcmp(computer,'PCWIN')
   save slicotin.mat A B C D -V4
elseif strcmp(computer,'SGI')
   save slicotin.mat A B C D
end

% save parameters on the mtb03ad.dat file for slicot subroutine
fid = fopen('mtb03ad.dat','w');
fprintf(fid,'%s \r\n','TB03AD DATA FILE GENERATED BY TB03AD.M');
fprintf(fid,'%4d %4d %4d %12.8f %2s %2s %4d %2s',size(A,1),size(B,2),size(C,1),TOL,LERI,EQUIL,OUTP,TYPE2);
fclose(fid);

% run the slicot subroutine
!MTB03AD.exe

% retrieve the results
load slicotou.mat

if TYPE == 's'
   Sys_elem = tf('s');
   sens = 1;
elseif TYPE == 'z'
   Sys_elem = tf('z');
   sens = 1;
elseif TYPE == 'l' | TYPE == 'q'
   Sys_elem = tf('z');
   sens = -1;
end

% Calculate scaling factor, if any
if SCALING > 0
   if LERI == 'L'
      Ps1=1;
      if SCALING == 1
         PQi=eval('P0');
         Ps2=1/max(max(abs(PQi)));
      elseif SCALING == 2
         n=0;
         Ps2=0;
         while exist(strcat('P',num2str(n))),
            PQi=eval(strcat('P',num2str(n)));
            Ps2=max(Ps2,max(max(abs(PQi))));
            n=n+1;
         end
         Ps2=1/Ps2;
      elseif SCALING == 3
         PQi=eval('P0');
         Ps2=inv(PQi);
      end
   else
      if SCALING == 1
         PQi=eval('P0');
         Ps1=1/max(max(abs(PQi)));
      elseif SCALING == 2
         n=0;
         Ps1=0;
         while exist(strcat('P',num2str(n))),
            PQi=eval(strcat('P',num2str(n)));
            Ps1=max(Ps1,max(max(abs(PQi))));
            n=n+1;
         end
         Ps1=1/Ps1;
      elseif SCALING == 3
         PQi=eval('P0');
         Ps1=inv(PQi);
      end
      Ps2=1;
   end
end

% Generate P polynomial = P0 + P1 s + P2 s^2 + ... {in s or in z or in z^-1 = lambda}
% Stores Pi matrices in TP array of matrices
P=0;
n=0;
while exist(strcat('P',num2str(n))),
   PQi=eval(strcat('P',num2str(n)));
   if SCALING > 0
      TP(:,:,n+1)=Ps1*PQi*Ps2;
   else
      TP(:,:,n+1)=PQi;
   end
   if FLAG == 'tf'
      P=P+TP(:,:,n+1)*Sys_elem^(sens*n);
   end
   n=n+1;
end
orderp=n-1;

% generate Q polynomial = Q0 + Q1 s + Q2 s^2 + ... {in s or in z or in z^-1 = lambda}
% Stores Qi matrices in TQ array of matrices
Q=0;
n=0;
while exist(strcat('Q',num2str(n))),
   PQi=eval(strcat('Q',num2str(n)));
   if SCALING > 0
      TQ(:,:,n+1)=Ps2*PQi*Ps1;
   else
      TQ(:,:,n+1)=PQi;
   end
   if FLAG == 'tf'
      Q=Q+TQ(:,:,n+1)*Sys_elem^(sens*n);
   end
   n=n+1;
end
orderq=n-1;

% display on screen
disp(sprintf('Order of Num = Q = %d',orderq));
disp(sprintf('Order of Den = P = %d',orderp));

if (FLAG == 'tf') & (TYPE == 'l' | TYPE == 'q')
   orderdiff=orderp-orderq;
   if orderdiff < 0
      P=P*Sys_elem^(orderdiff);
   else
      Q=Q*Sys_elem^(-orderdiff);
   end
   set(P,'Variable','z^-1')
   set(Q,'Variable','z^-1')
end

if FLAG == 'tf'
   Num = Q;
   Den = P;
else
   Num = TQ;
   Den = TP;
end

if CLEAN == 1
   if strcmp(computer,'PCWIN')
      !del slicotin.mat
      !del mtb03ad.dat
      !del slicotou.mat
      !del mtb03ad.res
   elseif strcmp(computer,'SGI')
      %!rm slicotin.mat
      %!rm mtb03ad.dat
      %!rm slicotou.mat
      %!rm mtb03ad.res
   end
end

